#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "x_text.hpp"
#include "unique.hpp"
#include "packet.hpp"
#include "select.hpp"

enum option_t { GAME, DIFFICULTY, EPISODE, MISSION, GAME_TYPE, NOMONSTERS, RESPAWN, TURBO, PWAD, N_OPTIONS };

char			*value[11] = { "Heretic   ", "3", "1", "1 ", "COOPERATIVE    ", "NO ", "NO ", "0", setup.pwad };
void			beep();
void			editValue(int option, int direction);
void			updateValue(int option);
void			hiliteValue(int option);
void			unhiliteValue(int option);

int			setupKnown;

void beep() {
  sound(1500);
  delay(5);
  nosound();
}

void editValue(int option, int direction) {
  int c, l, done = 0;
  switch(option) {
    case GAME:
      if (!direction) {
	setup.game--; if (setup.game < 0) setup.game = N_GAMES - 1;
      } else {
	setup.game++; if (setup.game >= N_GAMES) setup.game = 0;
      }
      sprintf(value[GAME], gameName[setup.game]);
      break;
    case DIFFICULTY:
      if (!direction) {
	gameCom.skill--; if (gameCom.skill < 1) gameCom.skill = 5;
      } else {
	gameCom.skill++; if (gameCom.skill > 5) gameCom.skill = 1;
      }
      sprintf(value[DIFFICULTY], "%d", gameCom.skill);
      break;
    case EPISODE:
      if (!direction) {
	gameCom.episode--; if (!gameCom.episode) gameCom.episode = 4;
      } else {
	gameCom.episode++; if (gameCom.episode > 4) gameCom.episode = 1;
      }
      sprintf(value[EPISODE], "%d", gameCom.episode);
      break;
    case MISSION:
      if (!direction) {
	gameCom.map--; if (!gameCom.map) gameCom.map = 32;
      } else {
	gameCom.map++; if (gameCom.map > 32) gameCom.map = 1;
      }
      sprintf(value[MISSION], "%d", gameCom.map);
      break;
    case GAME_TYPE:
      if (!direction) {
	gameCom.deathmatch--; if (gameCom.deathmatch < 0) gameCom.deathmatch = 2;
      } else {
	gameCom.deathmatch++; if (gameCom.deathmatch > 2) gameCom.deathmatch = 0;
      }
      switch(gameCom.deathmatch) {
	case 0: strcpy(value[GAME_TYPE], "COOPERATIVE"); break;
	case 1: strcpy(value[GAME_TYPE], "DEATHMATCH"); break;
	case 2: strcpy(value[GAME_TYPE], "DEATHMATCH v2.0"); break;
      }
      break;
    case NOMONSTERS:
      noMonsters = !noMonsters;
      if (noMonsters) strcpy(value[NOMONSTERS], "YES");
      else strcpy(value[NOMONSTERS], "NO");
      break;
    case RESPAWN:
      respawn = !respawn;
      if (respawn) strcpy(value[RESPAWN], "YES");
      else strcpy(value[RESPAWN], "NO");
      break;
    case TURBO:
      if (!direction) {
	setup.ticdup--; if (setup.ticdup < 2) setup.ticdup = 5;
      } else {
	setup.ticdup++; if (setup.ticdup > 5) setup.ticdup = 2;
      }
      sprintf(value[TURBO], "%d", setup.ticdup - 2);
      break;
    case PWAD:
      value[PWAD][0] = '\0';
      hiliteValue(PWAD);
      gotoxy(39, 15);
      _setcursortype(_SOLIDCURSOR);
      while (!done) {
	c = getch();
	beep();
	if (!c) {
	  c = getch();
	} else {
	  switch(c) {
	    case 8:
	      l = strlen(value[PWAD]);
	      if (l) {
		value[PWAD][l - 1] = '\0';
		hiliteValue(PWAD);
	      }
	      break;
	    case 13:
	      done = 1;
	      break;
	    default:
	      if (isprint(c) && strlen(value[PWAD]) < 31) {
		l = strlen(value[PWAD]);
		value[PWAD][l] = c;
		value[PWAD][l + 1] = '\0';
		hiliteValue(PWAD);
	      }
	      break;
	  }
	}
      }
      _setcursortype(_NOCURSOR);
      break;
  }
}

void updateValue(int option) {
  int x, y;
  gotoxy(39, 6 + option);
  cprintf(value[option]);
  textcolor(SCREEN_FG);
  textbackground(SCREEN_BG);
  x = wherex(); y = wherey();
  cprintf("                ");
  gotoxy(x, y);
}

void hiliteValue(int option) {
  textcolor(WHITE);
  textbackground(BLACK);
  updateValue(option);
}

void unhiliteValue(int option) {
  textcolor(LIGHTGRAY);
  textbackground(BLACK);
  updateValue(option);
}

void selectInit() {
  sprintf(value[TURBO], "%d", setup.ticdup - 2);
  textcolor(GREEN);
  textbackground(BLACK);
  gotoxy(33, 6); cprintf("Game:");
  gotoxy(27, 7); cprintf("Difficulty:");
  gotoxy(30, 8); cprintf("Episode:");
  gotoxy(30, 9); cprintf("Mission:");
  gotoxy(28, 10); cprintf("Game Type:");
  gotoxy(27, 11); cprintf("NoMonsters:");
  gotoxy(30, 12); cprintf("Respawn:");
  gotoxy(32, 13); cprintf("Turbo:");
  gotoxy(30, 14); cprintf("PWAD(s):");
  textcolor(RED); textbackground(BLACK);
  gotoxy(12, 19); cprintf("After you have made your selections, press");
  textcolor(WHITE);
  cprintf(" Alt-G ");
  textcolor(RED);
  cprintf("to begin");
  textcolor(RED); textbackground(BLACK);
  gotoxy(31, 21); cprintf("Press");
  textcolor(WHITE);
  cprintf(" Alt-X ");
  textcolor(RED);
  cprintf("to exit");
  for (int option = DIFFICULTY; option < N_OPTIONS; option++)
    unhiliteValue((option_t)option);
}

void selectOptions() {
  int c, done = 0, abort = 0, cur_option = GAME;
  _setcursortype(_NOCURSOR);
  selectInit();
  hiliteValue(cur_option);
  while (!done) {
    c = getch();
    if (!c) {
      c = getch();
      switch(c) {
	case 34:
	  done = 1;
	  break;
	case 45:
	  done = abort = 1;
	  break;
	case 72:
	  unhiliteValue(cur_option);
	  cur_option--; if (cur_option < 0) cur_option = N_OPTIONS - 1;
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 75:
	  editValue(cur_option, 0);
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 77:
	  editValue(cur_option, 1);
	  hiliteValue(cur_option);
	  beep();
	  break;
	case 80:
	  unhiliteValue(cur_option);
	  cur_option++; if (cur_option == N_OPTIONS) cur_option = GAME;
	  hiliteValue(cur_option);
	  beep();
	  break;
      }
    } else {
      if (c == 13) {
	editValue(cur_option, 1);
	if (cur_option != PWAD) hiliteValue(cur_option);
	beep();
      }
    }
  }
  _setcursortype(_NORMALCURSOR);
  window(1,2,80,25);
  textcolor(SCREEN_FG); textbackground(SCREEN_BG);
  clrscr();
  window(1,1,80,25);
  gotoxy(1,3);
  if (abort) {
    printf("Aborted...\n");
    exit(0);
  }
  setupKnown = 1;
}
